package cn.kdan.pdf.backend.core.service.impl;

import cn.kdan.pdf.backend.core.constant.SubscriptionConstant;
import cn.kdan.pdf.backend.core.enums.*;
import cn.kdan.pdf.backend.core.mapper.PricingDiscountsMapper;
import cn.kdan.pdf.backend.core.mapper.SubscriptionsMapper;
import cn.kdan.pdf.backend.core.model.*;
import cn.kdan.pdf.backend.core.pojo.SubscriptionVO;
import cn.kdan.pdf.backend.core.service.*;
import cn.kdan.pdf.backend.core.utils.PeriodUtils;
import enums.CommonEnum;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import utils.CommonUtils;

import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;


/**
 * @author tangxiangan
 */
@Slf4j
@Service
public class SubscriptionServiceImpl implements SubscriptionService {

    @Autowired
    private SubscriptionsMapper subscriptionsMapper;
    @Autowired
    private SetPricingService setPricingService;
    @Autowired
    private PricingService pricingService;
    @Autowired
    private MemberService memberService;
    @Autowired
    private OrderService orderService;
    @Autowired
    private PricingDiscountsMapper pricingDiscountsMapper;

    @Override
    public long countTimesById(String memberId, String discountId) {
        SubscriptionsExample example = new SubscriptionsExample();
        example.createCriteria().andStatusNotEqualTo(SubscriptionStatusEnum.UNPAYED.value()).andPricingDiscountIdEqualTo(discountId).andMemberIdEqualTo(memberId);
        return subscriptionsMapper.countByExample(example);
    }

    @Transactional(rollbackFor = BackendRuntimeException.class)
    @Override
    public SubscriptionVO create(String client, Integer payment, String targetType, String targetId) {
        //判断平台类型
        client = StringUtils.isEmpty(client) ? ClientEnum.APP.value() : client;
        if(!PaymentEnum.checkEnumByValue(payment)){
            throw new BackendRuntimeException(SubscriptionConstant.EXCEPTION_MSG_INVALID_PAYMENT);
        }
        if (GoodsTypeEnum.SetPricing.value().equals(targetType)) {
            List<SetPricings> list = setPricingService.getForSaleRMBById(targetId);
            return getSubscriptionPojo(client, payment, targetType, targetId, CollectionUtils.isEmpty(list));
        }else if(GoodsTypeEnum.Pricing.value().equals(targetType)){
            List<Pricings> list = pricingService.getForSaleRMBById(targetId);
            return getSubscriptionPojo(client, payment, targetType, targetId, CollectionUtils.isEmpty(list));
        }
        return null;
    }

    /**
     *
     * 查找用户没有没未支付的指定类型的订阅 如果没有则生成
     */
    public Subscriptions findMemberUnPayedSubscription(Members member, String targetType, Integer targetId,Integer payment,String client) {
        SubscriptionsExample example = new SubscriptionsExample();
        example.createCriteria().andMemberIdEqualTo(member.getId()).andTargetIdEqualTo(targetId).andPaymentEqualTo(payment).andTargetTypeEqualTo(targetType).andStatusEqualTo(SubscriptionStatusEnum.UNPAYED.value());
        List<Subscriptions> subscriptionsList = subscriptionsMapper.selectByExample(example);
        //如果该用户没有未支付的订阅记录,则插入一条
        if (CollectionUtils.isEmpty(subscriptionsList)) {
            if (GoodsTypeEnum.SetPricing.value().equals(targetType)) {
                SetPricings setPricings = setPricingService.getById(targetId);
                Subscriptions subscriptions = getSubscriptions(member, targetType, targetId, payment);
                // 计算订阅的开始时间和结束时间
                renewInfo(true,client,subscriptions,member,setPricings);
                subscriptionsMapper.insert(subscriptions);
                return subscriptions;
            }else if(GoodsTypeEnum.Pricing.value().equals(targetType)){
                Pricings pricings = pricingService.getById(targetId);
                Subscriptions subscriptions = getSubscriptions(member, targetType, targetId, payment);
                // 计算订阅的开始时间和结束时间
                renewInfo(true,client,subscriptions,member,pricings);
                subscriptionsMapper.insert(subscriptions);
                return subscriptions;
            }
        }
        return subscriptionsList.get(0);
    }

    /**
     * 获取订阅pojo对象
     * @param client 客户端（web、android）
     * @param payment 支付方式（0：支付宝、1：微信）
     * @param targetType 购买服务（SetPricing、Pricing）
     * @param targetId 服务id
     * @param empty 查询购买的服务是否为空
     * @return 订阅pojo
     */
    private SubscriptionVO getSubscriptionPojo(String client, Integer payment, String targetType, String targetId, boolean empty) {
        if (empty) {
            throw new BackendRuntimeException(SubscriptionConstant.EXCEPTION_MSG_INVALID_TARGET_ID);
        }
        Members members = memberService.getCurrentUser();
        //查找用户未支付状态的会员服务 没有则插入
        Subscriptions subscription = findMemberUnPayedSubscription(members, targetType, Integer.parseInt(targetId), payment, client);
        //查找用户有没有没有支付的微信订单，没有则插入(填充用户订单信息)
        Orders order = orderService.findMemberUnPayedOrders(members.getId(), subscription);
        SubscriptionVO pojo = new SubscriptionVO();
        BeanUtils.copyProperties(subscription, pojo);
        pojo.setOrder(order);
        return pojo;
    }

    /**
     * 对订阅对象赋值
     * @param member 用户
     * @param targetType 购买服务（SetPricing、Pricing）
     * @param targetId 服务id
     * @param payment 支付方式（0：支付宝、1：微信）
     * @return 订阅对象
     */
    private Subscriptions getSubscriptions(Members member, String targetType, Integer targetId, Integer payment) {
        Subscriptions subscriptions = new Subscriptions();
        subscriptions.setId(CommonUtils.generateId());
        subscriptions.setMemberId(member.getId());
        subscriptions.setTargetType(targetType);
        subscriptions.setTargetId(targetId);
        subscriptions.setStatus(SubscriptionStatusEnum.UNPAYED.value());
        subscriptions.setCreatedAt(new Date());
        subscriptions.setUpdatedAt(new Date());
        subscriptions.setPayment(payment);
        return subscriptions;
    }

    @Override
    public void renewInfo(Boolean force, String client, Subscriptions subscription, Members member, Object target) {
        client = StringUtils.isEmpty(client) ? "web" : client;
        //填充价格，总券数总时间
        discountInfo(subscription,client,member,target,subscription.getTargetType());
        if(StringUtils.isEmpty(subscription.getTargetId())){
            subscription.setStartDate(new Date());
            //开始时间加上结束时间
            subscription.setEndDate(!StringUtils.isEmpty(subscription.getTotalPeriod())? PeriodUtils.countDate(subscription.getStartDate(),subscription.getTotalPeriod()) : subscription.getStartDate());
        }else {

            Subscriptions lastSubs = getLastSub(subscription,member,target);
            //当前订阅不等于上一个订阅且上一个订阅的结束日期不为空时，设置当前订阅的开始日期为上一个订阅的结束日期加1秒；否则设置开始日期为当前时间
            if(!ObjectUtils.isEmpty(lastSubs)&&!subscription.getId().equals(lastSubs.getId()) && lastSubs.getEndDate() != null){
                LocalDateTime localDateTime = PeriodUtils.covertLocalDateTime(lastSubs.getEndDate());
                localDateTime.plus(1L, ChronoUnit.SECONDS);
                subscription.setStartDate(PeriodUtils.covertDate(localDateTime));
            } else{
                subscription.setStartDate(new Date());
            }

            /*
                1.调用countPeriod方法有无值、有则继续
                  sub中referral_id是否为空,不为空设置period为7.days
                  直接设置period为period或者countPeriod
                2.计算t_period值
                3.如果t_period大于等于1.days
                  endDate=（startDate+t_period）的一天开始减一秒（23.59.59）
                  否则endDate=startDate+t_period
             */
            PricingDiscounts pricingDiscounts = pricingDiscountsMapper.selectByPrimaryKey(subscription.getPricingDiscountId());

            String period;
            //进行period的赋值
            if(subscription.getTargetType().equals(CommonEnum.SubscriptionTypeEnum.SET_PRICING.value())){
                //获取setPricing对象
                SetPricings setPricing = (SetPricings) target;
                period = setPricingService.countPeriod(setPricing, pricingDiscounts);
            } else{
                //获取pricing对象
                Pricings pricing = (Pricings) target;
                period = pricingService.countPeriod(pricing, pricingDiscounts);
            }
            if(!StringUtils.isEmpty(period)){
                //进行period的赋值
                if(!StringUtils.isEmpty(subscription.getMemberReferralId())){
                    subscription.setTotalPeriod("7.days");
                } else{
                    subscription.setTotalPeriod(period);
                }
            }
            //设置endDate
            if(!StringUtils.isEmpty(period)) {
                List<String> periodList = Arrays.stream(period.split("\\.")).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(periodList)) {
                    String num = periodList.get(0);
                    String unit = periodList.get(1);
                    if ("days".equals(unit) && 1 <= Integer.parseInt(num)) {
                        // 取结束日期前一天的结束时间（23:59:59）
                        Date endDate = PeriodUtils.countDate(subscription.getStartDate(), period);
                        LocalDateTime localDateTime = PeriodUtils.covertLocalDateTime(endDate);
                        Date date = PeriodUtils.covertDate(localDateTime.minusSeconds(1));
                        subscription.setEndDate(date);
                    } else {
                        subscription.setEndDate(PeriodUtils.countDate(subscription.getStartDate(), period));
                    }
                }
            }
        }
    }

    /**
     * 判断是否有活跃折扣，设置订阅的折扣id，价格和总券数，时间
     * @param subscriptions 订阅
     * @param client 客户端
     * @param member 用户
     * @param target setpricing pricing对象
     * @param targetType  类型
     */
    private void discountInfo (Subscriptions subscriptions,String client,Members member,Object target, String targetType) {
        if (GoodsTypeEnum.SetPricing.value().equals(targetType)){
            SetPricings setPricing = (SetPricings)target;
            //如果有折扣id的话查询折扣id，如果没有折扣id则返回null
            PricingDiscounts activeDiscount = !StringUtils.isEmpty(setPricing.getPricingDiscountId()) ? setPricingService.getActiveDiscount(setPricing.getPricingDiscountId(),member): null;
            //如果有活跃折扣设置活跃折扣id
            subscriptions.setPricingDiscountId(!ObjectUtils.isEmpty(activeDiscount)? activeDiscount.getId() : null);
            subscriptions.setPrice(setPricingService.countPrice(setPricing,activeDiscount,member,client));
            subscriptions.setTotalPoints(setPricingService.countPointsAmount(setPricing,activeDiscount));
            subscriptions.setTotalPeriod(setPricingService.countPeriod(setPricing,activeDiscount));
        }else{
            // pricing
            Pricings pricing = (Pricings)target;
            //如果有折扣id的话查询折扣id，如果没有折扣id则返回null
            PricingDiscounts activeDiscount = !StringUtils.isEmpty(pricing.getPricingDiscountId()) ? setPricingService.getActiveDiscount(pricing.getPricingDiscountId(),member): null;
            //如果有活跃折扣设置活跃折扣id
            subscriptions.setPricingDiscountId(!ObjectUtils.isEmpty(activeDiscount)? activeDiscount.getId() : null);
            subscriptions.setPrice(pricingService.countPrice(pricing,activeDiscount,member,client));
            subscriptions.setTotalPoints(pricingService.countPointsAmount(pricing,activeDiscount));
            subscriptions.setTotalPeriod(pricingService.countPeriod(pricing,activeDiscount));
        }


    }

    /**
     * 获取上一个订阅记录
     * @param subscriptions 订阅
     * @param members 用户
     * @param obj setPricing或pricing
     * @return 上一个订阅记录
     */
    public Subscriptions getLastSub(Subscriptions subscriptions, Members members, Object obj){
        Integer targetId = subscriptions.getTargetId(); //会员服务id或券服务id
        if(ObjectUtils.isEmpty(members)){
            return subscriptions;
        }
        if(StringUtils.isEmpty(targetId)){
            return null;
        }
        Subscriptions lastSub = null;
        if(CommonEnum.SubscriptionTypeEnum.SET_PRICING.value().equals(subscriptions.getTargetType())){
            SetPricings setPricings = setPricingService.getById(targetId);
            if(SetPricingModeEnum.member_v2.value().equals(setPricings.getMode())){
                List<Subscriptions> list = getSubscriptionList(subscriptions,members);
                for (Subscriptions subs : list) {
                    if(lastSub != null){
                        return lastSub;
                    } else if(!ObjectUtils.isEmpty(subs) && setPricings.getMode().equals(SetPricingModeEnum.free.value())){
                        lastSub = subs;
                    }
                }
            }
        } else{
            SubscriptionsExample example = new SubscriptionsExample();
            example.createCriteria().andMemberIdEqualTo(members.getId()).andStatusEqualTo(SubscriptionStatusEnum.ACTIVED.value())
                    .andIdNotEqualTo(subscriptions.getId()).andTargetIdEqualTo(subscriptions.getTargetId());
            List<Subscriptions> list = subscriptionsMapper.selectByExample(example);
            if(!CollectionUtils.isEmpty(list)){
                lastSub = list.get(0);
            }
        }
        return lastSub;
    }

    /**
     * 获取订阅记录集合
     * @param sub 订阅
     * @param members 用户
     * @return 订阅记录集合
     */
    public List<Subscriptions> getSubscriptionList(Subscriptions sub, Members members){
        SubscriptionsExample example = new SubscriptionsExample();
        example.createCriteria().andMemberIdEqualTo(members.getId())
                .andStatusEqualTo(SubscriptionStatusEnum.ACTIVED.value()).andTargetTypeEqualTo(sub.getTargetType())
                .andIdNotEqualTo(sub.getId());
        return subscriptionsMapper.selectByExample(example);
    }

    @Override
    public void update(Subscriptions subscription) {
        subscriptionsMapper.updateByPrimaryKeySelective(subscription);
    }

    @Override
    public Subscriptions getById(String subscriptionId) {
        return subscriptionsMapper.selectByPrimaryKey(subscriptionId);
    }
}
